Package: decoder
Type: Package
Title: Decode Coded Variables to Plain Text and the Other Way Around
Version: 1.2.3
Authors@R: 
  person(
    "Erik", 
    "Bulow", 
    email   = "eriklgb@gmail.com",
    role    = c("aut", "cre"),
    comment = c(ORCID = "0000-0002-9973-456X")
  )
Description: Main function "decode" is used to decode coded key values to plain
    text. Function "code" can be used to code plain text to code if there is a
    1:1 relation between the two. The concept relies on 'keyvalue' objects used
    for translation. There are several 'keyvalue' objects included in the areas
    of geographical regional codes, administrative health care unit codes,
    diagnosis codes and more. It is also easy to extend the use by arbitrary 
    code sets.
License: GPL-2
Depends: R (>= 4.1.0)
Suggests: dplyr, DT, testthat, knitr, rmarkdown, tidyr
VignetteBuilder: knitr
Encoding: UTF-8
URL: https://github.com/eribul/decoder,
        https://eribul.github.io/decoder/
BugReports: https://github.com/eribul/decoder/issues
LazyData: TRUE
Language: en-US
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-15 10:07:29 UTC; xbuler
Author: Erik Bulow [aut, cre] (ORCID: <https://orcid.org/0000-0002-9973-456X>)
Maintainer: Erik Bulow <eriklgb@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-15 12:30:02 UTC
Built: R 4.5.2; ; 2026-05-15 13:32:50 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(as.keyvalue,data.frame)
S3method(as.keyvalue,default)
S3method(as.keyvalue,keyvalue)
S3method(as.keyvalue,list)
S3method(decode,data.frame)
S3method(decode,default)
S3method(summary,keyvalue)
export(as.key)
export(as.keyvalue)
export(as.value)
export(code)
export(decode)
export(is.key)
export(is.keyvalue)
export(is.value)
export(kungalv2Fyrbodal)
export(kungalv2Storgoteborg)
export(real_names)
export(short_region_names)

[![CRAN_Status_Badge](http://www.r-pkg.org/badges/version/decoder)](https://cran.r-project.org/package=decoder)
[![Build status](https://ci.appveyor.com/api/projects/status/63pi6rykfpsuru1c/branch/master?svg=true)](https://ci.appveyor.com/project/eribul/decoder/branch/master)
![Monthly downloads](http://cranlogs.r-pkg.org/badges/decoder) 
![Total downloads](http://cranlogs.r-pkg.org/badges/grand-total/decoder)
[![Project Status: Active - The project has reached a stable, usable state and is being actively developed.](http://www.repostatus.org/badges/latest/active.svg)](http://www.repostatus.org/#active)

decoder
==========


The goal of decoder is to decode coded and perhaps cryptical variables to more understandable descriptive labels.

## Installation

You can install the released version of decoder from [CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("decoder")
```

And the development version from [BitBucket](https://bitbucket.org/) with:

``` r
# install.packages("remotes")
remotes::install_bitbucket("cancercentrum/decoder")
```

[![CRAN_Status_Badge](http://www.r-pkg.org/badges/version/decoder)](https://cran.r-project.org/package=decoder)
[![Build status](https://ci.appveyor.com/api/projects/status/63pi6rykfpsuru1c/branch/master?svg=true)](https://ci.appveyor.com/project/eribul/decoder/branch/master)
![Monthly downloads](http://cranlogs.r-pkg.org/badges/decoder) 
![Total downloads](http://cranlogs.r-pkg.org/badges/grand-total/decoder)
[![Project Status: Active - The project has reached a stable, usable state and is being actively developed.](http://www.repostatus.org/badges/latest/active.svg)](https://www.repostatus.org/#active)

# decoder

The goal of decoder is to decode coded and perhaps cryptical variables to more understandable descriptive labels.

## Installation

You can install the released version of decoder from [CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("decoder")
```

And the development version from [GitHub](https://github.com/eribul/decoder/) with:

``` r
# install.packages("pak")
pak::pkg_install("eribul/decoder")
```

## -----------------------------------------------------------------------------
library(decoder)

## ----echo = FALSE, message=FALSE----------------------------------------------
library(decoder)
suppressPackageStartupMessages(library(dplyr))

sjukhus_all <-
  suppressWarnings(
    bind_rows(
      rockan = as_tibble(sjukhus),
      inca   = as_tibble(sjukhus_inca),
      par    = as_tibble(sjukhus_par),
      .id    = "source"
    )
  )

key_n_sources <- 
  sjukhus_all |> 
  count(key, sort = TRUE)

## ----echo = FALSE, message = FALSE--------------------------------------------

key_n_sources <- 
  sjukhus_all |> 
  count(key, sort = TRUE)
in3 <- key_n_sources |> filter(n == 3)

sjukhus_all |> 
  inner_join(in3, "key") |> 
  arrange(key) |> 
  tidyr::spread(source, value) |> 
  select(-n) |> 
  knitr::kable()

#' Coerce to a keyvalue object
#'
#' Functions to check if an object is a keyvalue object, or coerce it if
#' possible.
#'
#' \code{x} can be of: \itemize{ \item a name of a package internal (or
#' external) keyvalue object (character of length one) \item a \code{data.frame}
#' with two columns "key" (with unique cells) and "value". \item a list as
#' described in \code{\link{as.keyvalue.list}} \item a named vector with all
#' names unique \item an unnamed vector with unique cells if supplemented by
#' additional vector (y) }
#'
#' @name keyvalue
#' @param x object to test for, or coerce to, keyvalue (see the details).
#' @param y a value vector if \code{x} is an unnamed vector (argument must be
#'   named 'y = .')
#' @param standard_var_names a character vector with standard names for
#'   variables decoded with this key-values.
#' @param ... arguments passed between methods
#'
#' @return \code{as.keyvalue} returns a data.frame with additional class
#' \code{keyvalue} that fulfills the requirements for a \code{keyvalue} object
#' and with the \code{key} column as character. It also has two extra attributes
#' \code{standard_var_names} as described above and \code{keyvalue11} which is
#' \code{TRUE} for keyvalue objects with a 1:1 relation between its keys and
#' values (otherwise FALSE).
#'
#' @export
#' @seealso \link{as.keyvalue.list}, \link{decode}, \link{summary.keyvalue},
#'   \link{key_and_value}
#' @examples
#' snomed2 <- as.keyvalue(snomed)
#' is.keyvalue(snomed2)
#' summary(snomed2)
as.keyvalue <- function(x, ...){
    UseMethod("as.keyvalue")
}

context("as.keyvalue.list")

ex <- list(
  fruit  = c("banana", "orange", "kiwi"),
  car    = c("SAAB", "Volvo", "taxi", "truck"),
  animal = c("elefant")
)

# An invalid list (non unique names; not because of silly classification)
ex2 <- list(
  fruit  = c("banana", "orange", "kiwi"),
  car    = c("SAAB", "Volvo", "taxi", "truck"),
  animal = c("elefant"),
  fruit  = c("President_Obama")
)

# An invalid list (non unique keys; kiwi appear twice)
ex3 <- list(
  fruit  = c("banana", "orange", "kiwi"),
  car    = c("SAAB", "Volvo", "taxi", "truck"),
  animal = c("elefant", "kiwi")
)

test_that("misc", {
  expect_is(as.keyvalue.list(ex), "keyvalue")
  expect_error(as.keyvalue.list(ex2), 
    "All list element names must be unique!")
  expect_error(as.keyvalue.list(ex3), "Some key\\(s\\) have duplicates!")
  expect_error(as.keyvalue(list(1:10)), "All elements of x must be named")
  expect_error(as.keyvalue(list(a = list(1:10))), 
    "All elements of the list should be atomic!")
})

context("as.keyvalue.default")

test_that("misc", {
  expect_error(as.keyvalue.default(1), 
    "All elements of vector 'x' must be named!")
  expect_is(as.keyvalue.default(1, "hej"), "keyvalue")
  expect_error(as.keyvalue.default(1:10, letters), 
    "arguments imply differing number of rows")
  expect_error(as.keyvalue(1:10), "All elements of vector 'x' must be named!")
})

# Geography ---------------------------------------------------------------

#' Swedish health care regional codes
#'
#' Key-value codes for the Swedish health care regions (1-6).
#'
#' @docType data
#' @keywords datasets
#' @name region
#' @family key_value_data
NULL


#' Swedish county codes
#'
#' Key-value codes for the Swedish counties (lan).
#' Two first digits of the LKF code.
#' Be aware of the spelling ("lan")!
#'
#' @docType data
#' @keywords datasets
#' @name lan
#' @family key_value_data
NULL


#' Swedish district codes
#'
#' Key-value codes for the Swedish districts (introduced 2016-01-01).
#'
#' @docType data
#' @keywords datasets
#' @name distrikt
#' @family key_value_data
#' @source \url{https://www.scb.se/hitta-statistik/regional-statistik-och-kartor/regionala-indelningar/distrikt/}
NULL


#' Geographical health care areas
#'
#' Key-value codes for health care areas.
#'
#' \itemize{
#'   \item{Key: the first four digits from the LKF-code}
#'   \item{Value: A geographical area (sub area of county/lan) with special
#'     interest}
#' }
#'
#' The object currently only applies to the Western health care region
#' \itemize{
#'   \item{Storgoteborg}
#'   \item{Fyrbodal}
#'   \item{Sodra_Alvsborg}
#'   \item{Skaraborg}
#'   \item{Norra_Halland}
#' }
#' Updates for other regions needs to be requested (please do!).
#'
#' @section Kungalv:
#'  Kungalv is an independent area by default. There are situations when Kungalv
#'  should be dealt with as an
#'  independent health care region or as a part of greater Gothenburg.
#'  See section "extra_functions" in \link{decode}
#'  to handle this.
#'
#' @docType data
#' @keywords datasets
#' @name sjukvardsomrade
#' @seealso  \link{extra_functions}
#' @family key_value_data
NULL


#' HSN code (Hälso- och sjukvårdsnamnd)
#'
#' Key-value codes for the HSN:s in VGR valid from 2015.
#' Based on municipality codes (\code{\link{kommun}}).
#' Data from 2014-12-17.
#'
#' Categorization:
#' \describe{
#' \item{Norra}{Lysekil, Munkedal, Orust, Sotenäs, Strömstad, Tanum, Bengtsfors,
#' Dals-Ed, Färgelanda, Mellerud, Åmål, Trollhättan, Uddevalla och Vänersborg}
#' \item{Västra}{Lilla Edet, Ale, Kungälv, Stenungsund, Tjörn, Öckerö, Härryda,
#' Mölndal, Partille, Lerum och Alingsås}
#' \item{Södra}{Herrljunga, Vårgårda, Bollebygd, Borås, Mark, Svenljunga,
#' Tranemo och Ulricehamn}
#' \item{Göteborg}{Göteborg}
#' \item{Östra}{Essunga, Falköping, Grästorp, Götene, Lidköping, Skara, Vara,
#' Gullspång, Hjo, Karlsborg, Mariestad, Skövde, Tibro, Tidaholm och Töreboda}
#' }
#' @docType data
#' @keywords datasets
#' @name hsn
#' @family key_value_data
NULL


#' Swedish municipality codes
#'
#' Key-value codes for the Swedish municipalities.
#' First four digits in the LKF code.
#' Data from 2014-08-12.
#'
#' @docType data
#' @keywords datasets
#' @name kommun
#' @family key_value_data
#' @source \url{https://www.scb.se/hitta-statistik/regional-statistik-och-kartor/regionala-indelningar/lan-och-kommuner/}
NULL


#' Swedish parish codes
#'
#' Key-value codes for the Swedish parishes (forsamlingar).
#' All six digits in the LKF code.
#'
#' @docType data
#' @keywords datasets
#' @name forsamling
#' @family key_value_data
#' @source \url{https://www.scb.se/hitta-statistik/regional-statistik-och-kartor/regionala-indelningar/lan-och-kommuner/}
NULL


#' hemort and hemort2 codes (geographical codes)
#'
#' Both \code{hemort} and \code{hemort2} combines regional codes for lan,
#' kommun and forsamling.
#' \code{hemort} is the official (but rather old) code used within RCC.
#' \code{hemort2} is an updated version combining \code{\link{lan}},
#' \code{\link{kommun}} and \code{\link{forsamling}}
#'
#' @docType data
#' @keywords datasets
#' @name hemort
#' @family key_value_data
NULL

#' @rdname hemort
#' @name hemort2
NULL


# Administration ----------------------------------------------------------

#' Hospital codes
#'
#' Key-value codes for Swedish hospitals and some primary health care units.
#'
#' Data from 'Rockan'. Note that this is an old version of the classification!
#' See the reference link below for the new version.
#' The old version should be used of historical reasons.
#' One difference is for example that the Sahlgrenska university hospital is
#' one hospital
#' in the new version of the classification but the Swedish regional cancer
#' centers
#' still classify it as three different hospitals.
#' Note also that primary health care units did receive their own codes until
#' 1992 (?)
#' but not later!
#'
#' @docType data
#' @keywords datasets
#' @name sjukhus
#' @family key_value_data

NULL


#' Hospital codes used by Socialstyrelsen and the National Patient Register
#'
#' Codes taken from Excel sheets 2014 from link below.
#'
#' @docType data
#' @keywords datasets
#' @name sjukhus_par
#' @family key_value_data

NULL


#' Hospital codes used by INCA
#'
#' Codes taken from INCA's organizational register 2017-02-03.
#'
#' @docType data
#' @keywords datasets
#' @name sjukhus_inca
#' @family key_value_data
NULL


#' Clinic codes
#'
#' Key-value codes for the Swedish clinic types.
#' Data mainly from url below (taken from the PDF).
#' Some extra codes are added from Rockan
#' (290, 291, 292, 446, 921 and 999 for unknown).
#'
#' @docType data
#' @keywords datasets
#' @name klinik
#' @family key_value_data
NULL


#' Pathology department codes
#'
#' Key-value codes for the Swedish "laboratories".
#' Data is combined from two sources.
#' \enumerate{
#'  \item The official list from SOFS 2006:15 (see url below).
#'  This is the primary source for codes
#'  appearing in both sources.
#'  \item Extra codes are also added from the old Rockan registry for historical
#'  reasons.
#' }
#' @docType data
#' @keywords datasets
#' @name patologiavdelning
#' @family key_value_data
NULL


# Diagnostics -------------------------------------------------------------

#' Klassifikation av vardatgarder (KVA)
#'
#' Key-value codes for KVA (for surgery and medicine).
#'
#' @docType data
#' @keywords datasets
#' @name kva
#' @family key_value_data
NULL


#' M-stadium
#'
#' Key-value codes for M-stadium (sep-03).
#' Data from Rockan (Rocen).
#'
#' @docType data
#' @keywords datasets
#' @name m_rtr
#' @family key_value_data
NULL


#' N-stadium
#'
#' Key-value codes for N-stadium (sep-03).
#' Data from Rockan (Rocen).
#'
#' @docType data
#' @keywords datasets
#' @name n_rtr
#' @family key_value_data
NULL


#' T-stadium
#'
#' Key-value codes for T-stadium (sep 2003).
#' Data from Rockan (Rocen).
#'
#' @docType data
#' @keywords datasets
#' @name t_rtr
#' @family key_value_data
NULL


#' FIGO-stadium
#'
#' Key-value codes for FIGO-stadium (Tumorutbredning enl FIGO, sep 2003).
#' Data from Rockan (Rocen).
#'
#' @docType data
#' @keywords datasets
#' @name figo
#' @family key_value_data
NULL


#' ICD-7 Grov
#'
#' Key-value codes for just the first three digits of the ICDO-7 code.
#' This gives broader grouping.
#'
#' @docType data
#' @keywords datasets
#' @name icd7_grov
#' @family key_value_data
NULL


#' ICD-7
#'
#' Key-value codes for ICD-7.
#' Data from Rockan (Rocen).
#'
#' @docType data
#' @keywords datasets
#' @name icd7
#' @family key_value_data
NULL


#' ICD-9
#'
#' Key-value codes for ICD-9.
#' Data from Rockan (Rocen).
#'
#' @docType data
#' @keywords datasets
#' @name icd9
#' @family key_value_data
NULL


#' ICD-O
#'
#' Key-value codes for ICD-O.
#' Data from Rockan (Rocen).
#'
#' @docType data
#' @keywords datasets
#' @name icdo
#' @family key_value_data
NULL


#' ICD-O3
#'
#' Key-value codes for ICD-O3.
#' Data from Rockan (Rocen).
#'
#' @docType data
#' @keywords datasets
#' @name icdo3
#' @family key_value_data
NULL


#' ICD-O3 Grov
#'
#' Key-value codes for just the first two digits of the ICDO-3 code.
#' This gives broader grouping.
#' Based on the table of contents from a PDF.
#'
#' @docType data
#' @keywords datasets
#' @name icdo3_grov
#' @family key_value_data
NULL


#' Sida
#'
#' Key-value codes for sida (Sida, 1 = Hoger, 2 = Vanster, 9 = Okant).
#' Data from Rockan (Rocen).
#'
#' @docType data
#' @keywords datasets
#' @name sida
#' @family key_value_data
NULL


#' Grund till TNM (patologisk/klinisk)
#'
#' Key-value codes for TNM-grund (Grund till TNM, 1=patologisk, 2=klinisk).
#' Data from Rockan (Rocen).
#'
#' @docType data
#' @keywords datasets
#' @name tnmgrund
#' @family key_value_data
NULL


#' ICD-10-SE code
#'
#' Key-value codes for ICD-10-SE 2026 (Swedish version).
#' Note that key codes are given without dots, i e C569, not C56.9.
#'
#' @docType data
#' @keywords datasets
#' @name icd10se
#' @family key_value_data
NULL


#' ICD-10-CM code
#'
#' Key-value codes for ICD-10-CM.
#' Note that key codes are given without dots, i e C569, not C56.9.
#'
#' @docType data
#' @keywords datasets
#' @name icd10cm
#' @family key_value_data
NULL

#' ICD-9-CM diagnosis and procedure codes
#'
#' Key-value codes for ICD-9-CM diagnostics (\code{icd9cmd}) and
#' procedure (\code{icd9cmp}) codes (version 32).
#'
#' @docType data
#' @keywords datasets
#' @name icd9cmd
#' @family key_value_data
NULL

#' @rdname icd9cmd
#' @name icd9cmp
NULL


#' Snomed code
#'
#' Key-value codes for Snomed. Data from Rockan 2012-10-05.
#' Note tht this variable is the same as \code{snomed} from Rockan.
#' It should not be confused with the later version \code{snomed3}!
#'
#' @docType data
#' @keywords datasets
#' @name snomed
#' @family key_value_data
NULL


#' Snomed 3
#'
#' Key-value codes for Snomed3 (fr 2005).
#' Data from Rockan (Rocen).
#'
#' @docType data
#' @keywords datasets
#' @name snomed3
#' @family key_value_data
NULL


#' PAD (C24) code
#'
#' Key-value codes for PAD (C24) Data from Rockan 2012-09-18.
#'
#' @docType data
#' @keywords datasets
#' @name pad
#' @family key_value_data
NULL


# Misc --------------------------------------------------------------------

#' Gender code (kon)
#'
#' Key-value codes for gender (1 = man (Male) and 2 = Kvinna (female)).
#' Be aware of the spelling ("kon")!
#'
#' @docType data
#' @keywords datasets
#' @name kon
#' @family key_value_data
NULL

#' Anatomical Therapeutic Chemical (ATC) Classification System codes
#'
#' Key-value codes for ATC from the Swedish Medical Products Agency.
#'
#' @docType data
#' @keywords datasets
#' @name atc
#' @family key_value_data
NULL

#' Help tables from Rockan
#'
#' Key-value codes from old help tables used with Rockan
#' (by Swedish regional cancer centers)
#' @docType data
#' @keywords datasets
#' @name rockan
#' @family key_value_data
#' @aliases avgm ben digr dodca manuell obd status manuell
NULL

#' @export
#' @rdname keyvalue
as.keyvalue.data.frame <- function(x, standard_var_names = NULL, ...){
  
  if (ncol(x) != 2 || !all((c("key", "value") %in% names(x)))) {
    stop("A keyvalue object must have exactly two columns: 'key' and 'value'!")
  }
  
  x <- x[order(x$key),]
  
  x$value <- as.value(x$value)
  x$key   <- as.key(x$key)
  
  class(x) <- c("keyvalue", "data.frame")
  attr(x, "standard_var_names") <- standard_var_names
  
  ## If there is a 1:1-relation between the key and the value, 
  ## this is declared in an extra attribute
  if (!inherits(try(as.key(x$value), silent = TRUE), "try-error")) {
    x$value  <- as.key(x$value)
    attr(x, "keyvalue11") <- TRUE
  } else{
    attr(x, "keyvalue11") <- FALSE
  }
  
  x
}

context("as.keyvalue.data.frame")

iris2 <- iris[, 1:2]
names(iris2) <- c("key", "value")
iris3 <- iris2[!duplicated(iris2$key),]

test_that("misc", {
  expect_error(as.keyvalue.data.frame(iris), 
    "A keyvalue object must have exactly two columns: 'key' and 'value'!")
  expect_error(as.keyvalue.data.frame(iris2), "All keys must be unique!")
  expect_is(as.keyvalue.data.frame(iris3), "keyvalue")
  expect_false(is.keyvalue11(as.keyvalue.data.frame(iris3)))
})

#'Extra functions to apply to decoded variables using decode
#'
#'See section "extra_functions" at the \link{decode} help page!
#'
#' @param x The decoded variable to be further processed. 
#' This argument should not be set by the user.
#' It is used only internally by the function from which it is called.
#' @name extra_functions
#' @export
kungalv2Storgoteborg <- function(x){
    x <- as.character(x)
    x[x == "Kungalv"] <- "Storgoteborg"
    x
}

#' @rdname extra_functions
#' @export
kungalv2Fyrbodal <- function(x){
    x <- as.character(x)
    x[x == "Kungalv"] <- "Fyrbodal"
    x
}

#' @rdname extra_functions
#' @export
real_names <- function(x){
    x <- as.character(x)
    x[x == "Kungalv"]        <- "Kung\u00E4lv"
    x[x == "Norra_Halland"]  <- "Norra Halland"
    x[x == "Sodra_Alvsborg"] <- "S\u00F6dra \u00C4lvsborg"
    x[x == "Storgoteborg"]   <- "Storg\u00F6teborg"
    x
}

#' @rdname extra_functions
#' @export
short_region_names <- function(x){
    gsub("Region ", "", as.character(x))
}

#' Coerce to key or value in objects in a key-value pair
#'
#' Functions to check if an object is a valid key or value in a keyvalue pair, 
#' or coerce it if possible.
#' @name key_and_value
#' @param x a vector
#' @return \code{as.value} returns a value-object (currently the vector itself 
#' but with class "value").
#' \code{as.key} returns a key-object (building on \code{value} but with unique 
#' entries and as character).
#' @export
as.value <- function(x) {
  stopifnot(is.atomic(x))
  structure(x, class = c("value", class(x))) 
}


#' @rdname key_and_value
#' @export
is.value <- function(x){
  inherits(x, "value")
}

context("as.value")

test_that("as.value", {
  expect_is(as.value(1:10), c("value", "integer"))
})

test_that("is.value", {
  expect_true(is.value(decoder::snomed$value))
  expect_true(is.value(decoder::snomed$key))
})

#' Coerce a list to keyvalue object
#'
#' \code{x} should be a list with properties described in section "Details".
#' 
#' \itemize{
#'    \item{All names of the list elements should be unique.}
#'    \item{All elements of the list should be named.}
#'    \item{All keys should be unique (a key should only be mappad to 
#'    one value).}
#'    \item{All elements of the list should be atomic vectors.}
#' }
#'
#' @param x a \code{list} with properties described in the details section.
#' @param  ... further arguments passed to \code{\link{as.keyvalue}}.
#' 
#' @return The function returns an object of class \code{keyvalue} 
#' (and \code{list}.
#' @export
#' @seealso \link{as.keyvalue}
#' @examples
#'
#' ex <- list(
#'  fruit  = c("banana", "orange", "kiwi"),
#'  car    = c("SAAB", "Volvo", "taxi", "truck"),
#'  animal = c("elephant")
#' )
#' as.keyvalue(ex)
#' is.keyvalue(ex)
#'
as.keyvalue.list <- function(x, ...) {
  # Tests
  if (is.null(names(x)) || "" %in% names(x)) {
    stop("All elements of x must be named (values must be given for all keys)!")
  } else if (!identical(unique(names(x)), names(x))) {
    stop("All list element names must be unique!")
  } else if (anyDuplicated(unlist(x)) > 0) {
    stop(
      "Some key(s) have duplicates! A key should only be mapped to one ",
      "value (be found in one element of the list)"
    )
  } else if (!all(vapply(x, is.atomic, logical(1)))) {
    stop("All elements of the list should be atomic!")
  }
  
  as.keyvalue(internal_as.keyvalue.list(x), ...)
}


# Helper function 
internal_as.keyvalue.list <- function(x){
  data.frame(
    key       = unlist(x),
    value     = rep(names(x), vapply(x, length, numeric(1))),
    row.names = NULL,
    stringsAsFactors = FALSE
  ) 
}

#' Pad leading zeros to character
#' 
#' @param x character or object that can be coerced to such
#' @param n desired length of outcome vector
#'
#' @return Character vector of length \code{n}
pad0 <- function(x, n) {
  n <- pmax(0, n - nchar(x))
  paste0( 
    vapply(n, function(l) paste(rep("0", l), collapse = ""), character(1)),
    as.character(x))
}

# Try to transform x in order to fit the key from keyvalue
format_as_key <- function(x, keyvalue){
  
  x <- as.character(x)
  keyvalue <- as.keyvalue(keyvalue)
  
  # initiate warning message
  msg <- ""
  
  has     <- function(x, pattern) any(grepl(pattern, x), na.rm = TRUE)
  start_0 <- function(y) any(startsWith(y, "0"), na.rm = TRUE) 
  
  # Trim spaces 
  if (has(x, "[[:blank:]]")) {
    x <- trimws(x)
    msg <- "Spaces are removed from beginning and end."
  }
  
  # If x starts with zero but the key does not, zeroes are removed
  # This applies even if x is still character 
  # (and is therefore not an ifelse statement)
  if (start_0(x) & !start_0(keyvalue$key)) {
    msg <- paste(msg, "Leading 0:s are ignored.")
  }
  while (start_0(x) & !start_0(keyvalue$key)) {
    x <- 
      ifelse(
        startsWith(x, "0"), # If leading 0 ...
        substring(x, 2),    # ... remove the 0 ...
        x)                  # ... otherwise leave unchanged
  }
  
  # If key starts with 0 but x does not, pad with zeros!
  # Only applies if all codes have the same length and x is shorter than 
  # this length
  key_length <- nchar(keyvalue$key)[1]
  if (!start_0(x) && 
      start_0(keyvalue$key) && 
      Reduce(`==`, stats::na.omit(nchar(keyvalue$key))) &&
      key_length > min(nchar(x), na.rm = TRUE)) {
    x <- pad0(x, key_length)
    msg <- paste(msg, "Leading 0:s are introduced for short characters.")
  }
  
  # If x has punctuations but the key has not, punctuations are removed from x
  if (has(x,             "[[:punct:]]") && 
      !has(keyvalue$key, "[[:punct:]]")) {
    x <- gsub("[[:punct:]]", "", x)
    msg <- paste(msg, "Punctuations are removed.")
  }
  
  # If key has no capitals, all letters in x are made lowercase
  if (has(x,             "[[:upper:]]") && 
      has(keyvalue$key,  "[[:lower:]]") && 
      !has(keyvalue$key, "[[:upper:]]")) {
    x <- tolower(x)
    msg <- paste(msg, "Capitals are changed to lower case.")
    
    # If key has no lowercase, all letters in x are made upper case
  } else if (
    has(x,                "[[:lower:]]") && 
    has(keyvalue$key,     "[[:upper:]]") && 
    !has(keyvalue$key,    "[[:lower:]]")) {
    x <- toupper(x)
    msg <- paste(msg, "Lower case are changed to upper case.")
  }
  
  # If the key is shorter than x, x is substringed
  key_length <- max(nchar(keyvalue$key), na.rm = TRUE)
  if (max(nchar(x), na.rm = TRUE) > key_length) {
    x <- substr(x, 1, key_length)
    msg <- paste(msg, "Only the first", key_length, "characters are used.")
  }
  
  ## Print warning if any changes made
  if (msg != "")
    warning("transformed to match the keyvalue: ", msg, call. = FALSE)
  
  list(x = x, keyvalue = keyvalue)
}

context("formt_as_key")

test_that("lkf", {
  suppressWarnings(
    expect_true(all(nchar(format_as_key(forsamling$key, "lan")$x) <= 2)))
  expect_warning(format_as_key(forsamling$key, "lan"),
    "Only the first 2 characters are used.")
  suppressWarnings(
    expect_true(all(nchar(format_as_key(forsamling$key, "kommun")$x) <= 4)))
  expect_true(all(nchar(format_as_key(forsamling$key, "forsamling")$x) <= 6))
  expect_warning(format_as_key(".01", "lan"),
    "Punctuations are removed.")
  expect_warning(format_as_key("A", c("b" = "a")),
    "Capitals are changed to lower case.")
  expect_warning(format_as_key("a", c("B" = "A")),
    "Lower case are changed to upper case.")
  expect_warning(format_as_key(1, c(a = "01")),
    "Leading 0:s are introduced for short characters.")
})

#' Object summary for keyvalue objects
#'
#' Objects of type \code{keyvalue11} are summarized as
#' data.frames with an additional display of possible
#' \code{standard_var_names}. Objects of type \code{keyvalue}
#' also gives a list of non 1:1 key-value pairs.
#' @param object a keyvalue object
#' @param ... ignored
#' @return A list with three elements: a summary of the underlying data.frame,
#' the standard_var_names and possibly a data.frame with all non 1:1 relations.
#' @export
#' @seealso \link{keyvalue}
summary.keyvalue <- function(object, ...) {
  data.frame.summary <- summary.data.frame(object)

  standard_var_names <- attr(object, "standard_var_names")

  if (!is.keyvalue11(object)) {
    nonunique <-
      object[
        object$value %in%
          unlist(dimnames(table(object$value)[table(object$value) > 1])),
      ]
    nonunique <- nonunique[order(nonunique$value), ]
  }

  list(
    data.frame.summary = data.frame.summary,
    standard_var_names = standard_var_names,
    nonunique = nonunique
  )
}

#' @rdname decode
#' @export
code <- function(y, keyvalue, verbose = TRUE){
  
  if (is.character(keyvalue)) {
    keyvalue <- get(keyvalue)
  }
  
  ## If there is a no 1:1 relation between key and value,
  ## the process continues just if the particular translation 
  ## can be done safely!
  if (!is.keyvalue11(keyvalue)) {
    dangerous_values <- summary(keyvalue)$nonunique$value
    not11 <- y %in% dangerous_values
    if (any(not11)) {
      if (verbose) { 
        print(summary(keyvalue)$nonunique[dangerous_values %in% y, ])
        stop("Values above have a non unique relation to their key!")
      } else{
        stop("Some values have a non 1:1 relation to their key. ",
             "Use parameter verbose = TRUE to see a list!")
      }
    }
  }
  
  names(keyvalue) <- rev(names(keyvalue))
  decode(y, keyvalue, exact = TRUE)
}

context("code")

test_that("doesn't work", {
  expect_error(code(decoder::snomed$value, "snomed", verbose = FALSE), 
    "Some values have a non 1:1 relation to their key.")
  expect_error(capture_output(code(decoder::snomed$value, "snomed")), 
    "Values above have a non unique relation to their key!")
})

test_that("works", {
  expect_equal(code(kon$value, "kon"), as.character(1:2))
})

#'Decode codes to plain text (and vice versa)
#'
#'Translate coded values into meaningful plain text (or reversed).
#'
#'@param x object to decode. Either a key vector to be matched against the
#'  \code{key} column in \code{keyvalue}, or a data.frame (see section
#'  \code{decode.data.frame}). object.
#'@param y value to be coded (to be matched against the \code{value}
#' element) in a \link{keyvalue} object.
#'@param keyvalue either a name (as character string) of a package internal
#'  \link{keyvalue} object, or a user
#'  defined keyvalue object (see \link{as.keyvalue}).
#'@param exact Should \code{x} have an exact match from the key? Default is
#'  \code{FALSE}. When \code{FALSE}, \code{x} might be transformed to fit the
#'  key (punctuation might be removed, upper case changed to lower case or vice
#'  versa and strings that are too long might be substringed). (\code{code} only
#'  accept exact matches.)
#'@param extra_functions is a list of functions (or names of functions as
#'  character vector) to be applied to the decoded data after decoding (see
#'  section "extra_functions" below).
#'@param verbose (only for \code{code}) can be set to \code{FALSE} to avoid a
#'  printed message to the console if an error occur (\code{TRUE} as default).
#'@param ... ignored
#'
#'@return \itemize{ \item For default S3 method: A vector of the same length as
#'  \code{x} but with all cells decoded (or coded) to plain text (or code) as
#'  character. \item For S3 method for class 'data.frame': Data.frame \code{x}
#'  is returned, possibly with some extra columns (names ending in
#'  '_Beskrivning'), decoded from columns with names corresponding to attribute
#'  \code{standard_var_names} for keyvalue objects listed by
#'  \code{list_keyvalues()}. }
#'
#'@section Vignette: See the vignette for a longer introduction to the package:
#'  \code{vignette("decoder")}
#'
#'@section decode.data.frame: If \code{x} is a data.frame, all column names of
#'  \code{x} are matched to attribute \code{standard_var_names} for all keyvalue
#'  objects in the package (see \code{list_keyvalues()}). If the column name is
#'  a standard name used for a coding, the corresponding keyvalue object is used
#'  to decode the column and to add an extra column to \code{x} with its
#'  original name with suffix \code{_Beskrivning}. This is done for all
#'  identified columns.
#'
#'@section extra_functions: The relationship between the key and the value in a
#'  keyvalue object is either 1:1 or m:1. The mapping is straight forward for
#'  1:1 but with m:1, different applications might require slightly different
#'  groupings of the keys. One solution is to have several versions of the
#'  \link{keyvalue} object. Another (which we prefer) is to use the same
#'  \link{keyvalue} object (if possible) but to call one or several extra
#'  function(s) to further process the result. These functions are either built
#'  in package functions that should be called by quoted names or user defined
#'  functions that can be called by either quoted or unquoted names (if
#'  available in the current environment). Note that the order of the functions
#'  could matter since they are called in turn (the output from the first
#'  function is passed as input to the second function etc).
#'
#'  Standard functions and how to use them:
#'
#'  To use with \link{sjukvardsomrade}:
#'
#'  \describe{ \item{\code{kungalv2Fyrbodal}}{The default classification used in
#'  \link{sjukvardsomrade} is to make Kungalv a region of its own. Use this
#'  function if Kungalv should be included in Fyrbodal. See example section
#'  below.} \item{\code{kungalv2Storgoteborg}}{As \code{kungalv2Fyrbodal} but
#'  classifies Kungalv as a part of Storgoteborg.} \item{\code{real_names}}{Give
#'  the area names with correct Swedish spelling (including spaces). This is not
#'  as default due to compatibility reasons and because names with spaces must
#'  be back-ticked when referred to.} }
#'
#'  To use with \link{region} \describe{
#'  \item{\code{short_region_names}}{Exclude the prefix 'Region' from the region
#'  names, hence 'Syd' instead of 'Region Syd' etcetera.} }
#'
#'@seealso \link{keyvalue}, \link{extra_functions}
#' @examples
#'
#' KON_VALUE <- sample(1:2, 20, replace = TRUE)
#' (kon <- decode(KON_VALUE, decoder::kon))
#' code(kon, decoder::kon)
#'
#' # Get a sample of Snomed-codes
#' snomed2 <- sample(decoder::snomed$key, 30, replace = TRUE)
#' # ... then decode them:
#' (snomed3 <- decode(snomed2, "snomed"))
#'
#'
#' # Health care regions can be defined in more than one way
#' # By default Kungalv define a region of its own:
#' set.seed(123456789)
#' healtcare_areas_west <-
#'   sample(unlist(decoder::sjukvardsomrade), 100, replace = TRUE)
#' (areas <- decode(healtcare_areas_west, "sjukvardsomrade"))
#' table(areas)
#'
#' # But if we want Kungalv to be a part of Storgoteborg
#' # (which is common practice for example with lung cancer data):
#' (areas2 <-
#'   decode(healtcare_areas_west, "sjukvardsomrade", "kungalv2Storgoteborg"))
#' table(areas2)
#'
#' # We can also combine several extra_functions if we for example
#' # also want the area names with correct Swedish spelling.
#' (areas3 <-
#'   decode(
#'     healtcare_areas_west,
#'     "sjukvardsomrade", c("kungalv2Storgoteborg", "real_names")
#'  )
#' )
#'
#'
#' # The region names can be both with and without prefix:
#' regs <- sample(6, 10, replace = TRUE)
#' decode(regs, "region") # With prefix
#' decode(regs, "region", "short_region_names") # without prefix
#'
#' # Note that only the first four digits of the LKF-code were used abowe?
#' # What if we use the full LKF-code?
#' lkfs <- sample(decoder::forsamling$key, 100, replace = TRUE)
#' decode(lkfs, "sjukvardsomrade")
#' # That work's just as fine when exact = FALSE (which it is by default).
#'
#' # decode can also be used for data.frames with recognised column names
#' d <- data.frame(
#'      kon = sample(1:2, 10, replace = TRUE),
#'      sex = sample(1:2, 10, replace = TRUE),
#'      lkf = sample(decoder::hemort$key, 10, replace = TRUE)
#'  )
#'  decode(d)
#' @author Erik Bulow
#' @name decode
#' @export
decode <- function(x, ...) UseMethod("decode")

context("extra_functions")

x <- sjukvardsomrade$key

test_that("kungalv2Storgoteborg", {
  expect_false(
    "Kungalv" %in% unique(decode(x, "sjukvardsomrade", "kungalv2Storgoteborg")))
})

test_that("kungalv2Fyrbodal", {
  expect_false(
    "Kungalv" %in% unique(decode(x, "sjukvardsomrade", "kungalv2Fyrbodal")))
})

test_that("real_names", {
  expect_equal(unique(decode(x, "sjukvardsomrade", "real_names")), 
    c("Norra Halland", "Storg\u00F6teborg", "Kung\u00E4lv", 
      "Fyrbodal", "S\u00F6dra \u00C4lvsborg", "Skaraborg"))
})


test_that("short_region_names", {
  expect_equal(decode(3:6, "region", "short_region_names"),
   c("Syd\u00F6stra", "Syd", "V\u00E4st", "Norr"))
})

context("decode")

set.seed(12345)

d <-
  data.frame(
    kon = sample(1:2, 10, replace = TRUE),
    sex = sample(1:2, 10, replace = TRUE),
    lkf = sample(decoder::hemort$key, 10, replace = TRUE)
  )

test_that("decode.default", {
  expect_equivalent(decode(1:2, "kon"), c("Man", "Kvinna"))
  expect_equal(length(decode(snomed$key, "snomed")), length(snomed$key))
  expect_warning(decode("      0000001", "kon"))
  suppressWarnings(expect_equal(decode("      0000001", "kon"), "Man"))
  suppressWarnings(
    expect_equal(
      decode(c(0002425, 2425, "df"), "kommun"),
      c("Dorotea", "Dorotea", NA)
    )
  )
  expect_warning(decode(c(1:6, letters[1:5]), "region"), "(a, b, c, d, e)")
  expect_warning(decode(c(1:6, letters[1:10]), "region"), "(10 cells)")
})


test_that("decode.data.frame", {
  suppressMessages(expect_equal(decode(iris), iris))
  suppressMessages(
    expect_equal(
      decode(data.frame(KON_VALUE = 1:2))$KON_VALUE_kon_beskrivning,
      c("Man", "Kvinna")
    )
  )
  suppressMessages(suppressWarnings(expect_equal(ncol(decode(d)), 10)))
  expect_message(
    decode(data.frame(KON_VALUE = 1, KON_VALUE_kon_beskrivning = "Man")),
    "seems to be already decoded"
  )
})

#' @export
#' @rdname keyvalue
as.keyvalue.default <- function(x, y, ...){
  
  stopifnot(is.atomic(x))
  if (!missing(y)) stopifnot(is.atomic(y))
  
  # Possibly get keyvalue object by name 
  if (length(x) == 1    && 
      is.character(x)   && 
      is.null(names(x)) && 
      missing(y)        && 
      exists(x, envir = as.environment("package:decoder"))
    )
    return(as.keyvalue(get(x, envir = as.environment("package:decoder"))))
  
  ## If only x supplied, x should be a named vector
  if (missing(y)) {
    if (is.null(names(x)) || any(names(x) == "")) {
      stop("All elements of vector 'x' must be named!")
    }
    y <- names(x)   
  }
  
  df <- data.frame(key = x, value = y, row.names = NULL)
  as.keyvalue(df, ...)
}

#' @export
#' @rdname decode
decode.data.frame <- function(x, ...){
  
  nms <- names(x)
  
  # If x has variables named "_Beskrivning/_Värde" and no 
  # "_beskrivning/_värde", 
  # "_Beskrivning" is used otherwise "_beskrivning
  any_name <- function(x) any(endsWith(x, nms))
  beskrivning <- if (
      (any_name("_Beskrivning")  || any_name("_V\u00E4rde")) && 
      (!any_name("_beskrivning") || !any_name("_v\u00E4rde"))
    ) "_Beskrivning" else "_beskrivning"
  
  # Create a table describing all variables to decode
  decode_table <- merge(
    data.frame(x.var = nms, key = tolower(nms), stringsAsFactors = FALSE),
    ALL_STANDARD_VAR_NAMES,
    by = "key"
  )
  
  # Return x unchanged if no variables to decode:
  if (nrow(decode_table) == 0) {
    message("No variables decoded!")
    return(x)
  }
  
  # Otherwise, make names for new columns
  decode_table$new_name <- 
    paste0(decode_table$x.var, "_", decode_table$value, beskrivning)
  
  # Do not override existing _beskivnings-variables!
  already_beskrivning <- 
    intersect(tolower(decode_table$new_name), tolower(nms))
  if (!identical(already_beskrivning, character(0))) {
    message("Variable(s) ", paste(already_beskrivning, collapse = ", "),
            " seems to be already decoded and therefore left unchanged.")
    decode_table <- 
      decode_table[!(tolower(decode_table$new_name) %in% already_beskrivning), ]
  }
  
  # Decode each column
  for (i in seq_len(nrow(decode_table))) {
    d <- decode_table[i, , drop = FALSE]
    tryCatch(
      x[[d$new_name]] <- decode(x[[d$x.var]], d$value),
      warning = function(msg) {
        warning(d$x.var, " -> ", d$new_name, ": ", 
                sub("simpleWarning: ", "", msg), call. = FALSE)
      },
      finally = 
        suppressWarnings(x[[d$new_name]] <- decode(x[[d$x.var]], d$value))
    )
  }
  
  # Inform about new columns
  if (!identical(decode_table$new_name, character(0))) {
    message("New decoded columns added: \n* ", 
            paste(decode_table$new_name, collapse = "\n* ") )
  } 
  
  x
}

#' @export
#' @rdname decode
decode.default <- function(x, keyvalue, extra_functions = NULL, 
                           exact = FALSE, ...) {
  
  stopifnot(is.atomic(x))
  if (all(is.na(x))) return(x)
  
  original_x <- x
  keyvalue <- as.keyvalue(keyvalue)
  
  if (!exact) {
    transformed <- format_as_key(x, keyvalue)
    x <- transformed$x
    keyvalue <- transformed$keyvalue
  }
  
  ## Translate key to value
  positions <- match(x, keyvalue$key)
  res <- keyvalue[positions, "value", drop = TRUE]
  
  ## Apply extra functions if given
  if (!is.null(extra_functions)) {
    for (fun in extra_functions) {
      FUN <- get0(fun, mode = "function")
      if (is.null(FUN)) FUN <- match.fun(fun)
      res <- FUN(res)
    }
  }
  
  # Inform if some values could not be translated
  # Return these values if no more than five
  failed <- original_x[is.na(res) & !is.na(x)]
  if (length(failed) > 0 ) {
    msg <- paste("Some codes could not be translated")
    msg2 <- 
      if (length(failed) <= 5) 
        paste(failed, collapse = ", ") 
      else 
        paste(length(failed), "cells")
    msg <- paste0(msg, " (", msg2, ")")
    warning(msg, call. = FALSE)
  }
  
  res
}

#' @rdname key_and_value
#' @export
as.key <- function(x){
  
  if (!is.atomic(x)) 
    stop("cannot coerce class ", deparse(class(x)), " to a key object")
  
  # We have the same requirements for key as for values but 
  # the key must also be unique and coerced to character.
  x <- as.value(as.character(x))
  if (anyDuplicated(x) != 0) stop("All keys must be unique!")
  
  structure(x, class = unique(c("key", class(x))))
}


#' @rdname key_and_value
#' @export
is.key <- function(x){
  inherits(x, "key")
}

context("as.key")

test_that("as.key", {
  expect_equal(as.character(as.key(1:10)), as.character(1:10))
  expect_is(as.key(1:10), c("key", "value", "character"))
  expect_error(as.key(c(1:10, 1:10)))
})


test_that("is.key", {
  expect_true(is.key(decoder::snomed$key))
  expect_false(is.key(decoder::snomed$value))
  expect_true(is.key(decoder::kon$value))
})

context("datasets_keyvalue")

test_that("correct classes",{
  
  ## Geography
  expect_is(region,            "keyvalue")
  expect_is(lan,               "keyvalue")
  expect_is(kommun,            "keyvalue")
  expect_is(forsamling,        "keyvalue")
  expect_is(distrikt,          "keyvalue")
  
  ## Administration
  expect_is(sjukhus,           "keyvalue")
  expect_is(klinik,            "keyvalue")
  expect_is(patologiavdelning, "keyvalue")
  
  ## Diagnoses
  expect_is(t_rtr,             "keyvalue")
  expect_is(n_rtr,             "keyvalue")
  expect_is(m_rtr,             "keyvalue")
  expect_is(figo,              "keyvalue")
  expect_is(tnmgrund,          "keyvalue")
  expect_is(sida,              "keyvalue")
  expect_is(icd7,              "keyvalue")
  expect_is(icd9,              "keyvalue")
  expect_is(icd10se,           "keyvalue")
  expect_is(icd10cm,           "keyvalue")
  expect_is(icdo,              "keyvalue")
  expect_is(icdo3,             "keyvalue")
  expect_is(snomed,            "keyvalue")
  expect_is(snomed3,           "keyvalue")
  expect_is(pad,               "keyvalue")
  
  ## Others
  expect_is(kon,               "keyvalue")
  
  ## keyvaluem1
  expect_is(sjukvardsomrade,   "keyvalue")
})

context("keyvalue_methods")

test_that("summary.keyvalue", {
  expect_is(summary(snomed), "list")
  expect_equal(length(summary(snomed)), 3)
  expect_equal(names(summary(snomed)), 
              c("data.frame.summary", "standard_var_names", "nonunique"))
  expect_equal(summary(snomed)$standard_var_names, "snomed")
  expect_is(summary(snomed)$nonunique, "keyvalue")
})

---
title: "Alternative hospital names"
author: "Erik Bulow"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Alternative hospital names}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r}
library(decoder)
```


# Bakgrund

Vad vi nedan menar med sjukhus omfattar i många fall även vårdcentraler och privatläkare. 

Paketet innehåller fyra olika kodverk för sjukhuskoder:

| keyvalue     |      Källa                    |  Antal                           |
|--------------|-------------------------------|---------------------------------:|
| sjukhus_inca | Organisationsregistret i INCA | `r nrow(sjukhus_inca)` |
| sjukhus      | Rockan                        | `r nrow(sjukhus)`      |
| sjukhus_par  | SOS Patientregistret          | `r nrow(sjukhus_par)`  |


# sjukhus_inca

Detta kodverk baseras på organisationsenhetsregistret i INCA. Här inkluderas både aktiva INCA-inrapporterande enheter, samt historiska eheter inlästa från historiska data.

Se `?sjukhus_inca` för mer info.

# sjukhus

Dessa koder hämtas från Rockan, det äldre systemet för cancer- och kvalitetsregister som historiskt använts av regionerna Väst och Norr.
Dessa koder var de första att inkluderas i paketet. Nyare enheter (exempelvis Angereds Närsjukhus i Göteborg) saknas i denna lista. 

Se `?sjukhus` för mer info.



# sjukhus_par

Koder som används i Socialstyrelsnes Patientregister. Denna förteckning avser endast sjukhus och endast de enheter som var aktuella 2014 (vilket var senaste året en sådan lista sammanställdes).

Se `?sjukhus_par` för mer info.



```{r, echo = FALSE, message=FALSE}
library(decoder)
suppressPackageStartupMessages(library(dplyr))

sjukhus_all <-
  suppressWarnings(
    bind_rows(
      rockan = as_tibble(sjukhus),
      inca   = as_tibble(sjukhus_inca),
      par    = as_tibble(sjukhus_par),
      .id    = "source"
    )
  )

key_n_sources <- 
  sjukhus_all |> 
  count(key, sort = TRUE)
```


# Samma kod kan ge olika klartext

Även om samma kod hanteras av fler än ett kodverk så kan namnet i klartext vara olika:

```{r, echo = FALSE, message = FALSE}

key_n_sources <- 
  sjukhus_all |> 
  count(key, sort = TRUE)
in3 <- key_n_sources |> filter(n == 3)

sjukhus_all |> 
  inner_join(in3, "key") |> 
  arrange(key) |> 
  tidyr::spread(source, value) |> 
  select(-n) |> 
  knitr::kable()
```

---
title: "Decode coded variables to plain text (and vice versa)"
author: "Erik Bulow"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Decode coded variables to plain text (and vice versa)}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---


# Main purpose of the package

The main purpose of this packgae is twofold:

1. To easily translate coded variables into plain text using standardised dictionaries.
2. To provide an infrastructure to create such dictionaries.

The first purpose is assumed to dominate and the package therefore includes several dictionaries compatible with data from INCA and Rockan.



# Example for "decode"


Assume we have a dataset with some (more or less) typical RCC-data:

```{r}
set.seed(12345)
n <- 6
s <- function(x) sample(x, n, replace = TRUE)

incadata <- 
  data.frame(
    KON_VALUE = s(1:2),
    region = 1:6,
    a_icdo3 = c("C446", "C749", "C159", "C709", "C475", "C320"),
    a_tstad = c("0", "1", "1a", "1b", "1c", "2")
  )
knitr::kable(incadata)

```


## Manually specified decodings

`decode` is a generic S3-function that accepts R-object of different kinds.
The default method takes an atomic vector with codes and a specification of how to translate these codes (see `?decode`).

```{r}
library(decoder)

decode(incadata$KON_VALUE, "kon")
decode(incadata$a_tstad, "t_rtr")
```

See `?key_value_data` for a list of all available keyvalue objects included in the package.




## The whole data.frame at once

There is also a method for data.frames, where translations are made automatically for columns with recognised names. 
```{r}
incadata_decoded <- decode(incadata)
knitr::kable(incadata_decoded)
```

All the original columns from incadata are preserved but are now accompanied by some additional columns with identical names suffixed by "_beskrivning".
("_Beskrivning" with capital "B" will be used if there are already some column names with suffixes "_Beskrivning" or "_Värde" in the data, otherwise lower case variable names are generally preffered in all the rcc packages).



### A note of caution

To automatically transform all columns of a data.frame can be "dangerous" if the data.frame includes a column with arbitrary data but with a name just coincidentally beeing recognised as a standard variable name. It is therefore recommended to use this function mostly in interactive mode and to always look at the message specifying which columns that were automatically detected. 

It should however also be noted that the variable names are hard coded and have to be excact (although case differnecies are ignored). Hence, the variable "a_icdo3" were not recognised (even though its name could have been easily matched by a regular expression: `grepl("icdo3", "a_icdo3")`.



## Several keyvalues for the same code

Note that the same coded variable might be decoded by more than one keyvalue-object, for example the "a_icdo3"-variable is automatically decoded by the "icdo3" keyvalue object (see `?icdo3`) with a dictionary from Rockan. ICD-O3 however, can also be seen as a "developed subset" of ICD10, wherefore:

```{r}
decode(incadata$a_icdo3, "icd10se")
```

"icd10", can also decode non oncological desease codes:
```{r}
decode(c("D448A", "T009", "F182", "S134C", "C131"), "icd10se")
```



## Exact versus non exact decoding

So far, we have looked only on cases where the code can be littaraly translated by the keyvalue object. There is however a third argument to the decode function, `exact`, which is `FALSE` by default. It is often useful to allow some automatic transformation for variables that we know are coded values but that are not exactly the same as in the keyvalue object. The most common application is probably when we want to extract the municipality names from an LKF-variable. This can obviosly be done by:
```{r}
x <- c(178405, 138408, 108202, 128706, 048005)
y <- as.numeric(substring(as.character(x), 1, 4))
decode(y, "kommun", exact = TRUE)
```

But it is also possible to use the original six digit code as is:
```{r}
decode(x, "kommun")
```

Note here that the last value decodes to `NA` in both cases. It would be preffered to have the original codes stored as character, since that preserves the leading 0 in "048005":
```{r}
decode("048005", "kommun")
```
This can also be taken care of by reading data through the `use_incadata` function (see `?incadata::use_incadata`), which always classifies numbers with leading zeros as characters (and not numeric) (but this will not be covered in this vignette).

Note that use of `exact = FALSE` (which is the default for convenience) always produce warnings when some transformations are done. It is always recommended to manually confirm that these transformations are in fact accurate. Unexpected results might otherwise occur!


## More than one decoded variable from the same code

Using non exact decoding, the same code might be simultaneously decoded by more that one keyvalue object using the data.frame method:

```{r}
df <- 
  data.frame(
    LKF = c("149804", "147104", "012704", "143505", "126502", "232602")
  )
knitr::kable(suppressWarnings(decode(df)))
```


Consult `?hemort` and `?forsamling` to invest the differnce between these two variables (hint: a lot have happened with Swedish administrative geography since 1958).


## Extra functions

The atomic vector method has an additional argument `extra_functions` that can be used to modify the outcome of the decoded variable. Read the man page (`?decode`) for more details and a complete list of available extra functions.

The default regional names used from INCA are used by default by the "region" keyvalue:
```{r}
decode(1:6, "region")
```

Sometimes it is however more convenient to use an abbriviated form of these names such as:
```{r}
decode(1:6, "region", "short_region_names")
```

This could also be achieved by an additional keyvalue object containing the hard coded short names but an additional advantage with the "extra_function" method is that several extra functions can be chained:
```{r}
munic_west <- c("1382", "1419", "1441", "1460", "1472", "1488", "1496")
decode(munic_west, "sjukvardsomrade", c("kungalv2Storgoteborg", "real_names"))
```




# Example for "code" (the opposite of decode)

In the simplest situation (with a 1:1 relation between the code and its value), `code` is just the inverse of `decode`:
```{r}
code(c("Karlskrona", "Göteborg", "Härnösand"), "kommun")
```
`decode` does however also allow translation with m:1 dictionaries such as `snomed` (which gives a differnt result than `snomed3`):
```{r}
non_unique <- c(90703, 90723, 96153, 99403, 89643, 90443)
decode(non_unique, "snomed")
```

This decoding can not be inverted:
```{r, error = TRUE, purl = FALSE}
code(decode(non_unique, "snomed"), "snomed")
```
This restriction only applies when the requested values are de facto non unique. It is not tied to the keyvalue object as such.
```{r}
unique <- c(
  "Mucinöst kystadenom (kystom) borderline-typ ( 2 B)", 
  "Medullärt carcinom, atypiskt", 
  "Mb Paget non invasiv och intraduktal", 
  "Lymfangiosarcom, misst", 
  "Fibröst histiocytom, malignt", 
  "Ganglioneurom", 
  "Carcinoid (exkl appendix)", 
  "Langerhans cell histiocytos, UNS, Histiocytosis X, UNS", 
  "Brenner tumör, malign", 
  "Lymfatisk leukemi, UNS")
code(unique, "snomed")
```





# The underlying machinery

In all examples above the so called keyvalue object were specified by a quoted name (such as "`kon`" or "`region`" etcetera). These names refer to internal package objects that can not be lazy loaded from the package but which can still be accesed by the tripple colon notation:
```{r}
kon
region
```

These objects might look like ordinary data.frames but they do have some extra attributes as described by `?as.keyvalue`.
```{r}
attributes(kon)
```

The _keyvalue11_ attribute indicates that "code" can be used as an inverse of "decode" for translations using this keyvalue obejct, hence without the problems as described for `snomed`. 
The *standard_var_names* attribute provides a list with known names for the coded version of this variable when found in a data.frame (used by `decode.data.frame`).

A complete list of all standard variable names can also be found in `ALL_STANDARD_VAR_NAMES` (internal object):
```{r}
knitr::kable(head(decoder:::ALL_STANDARD_VAR_NAMES))
```

The key-column gives names of variables that are recognised and the value column gives the name of the corresponding keyvalue object to use for its decoding.




# Examples how to create keyvalue objects

It might sometimes come handy to create your own keyvalue objects for a specific project. The S3-generic `as.keyvalue` has several methods for this.

For small dictionaries, a named vector might be sufficient:
```{r}
(kv <- as.keyvalue(c("car" = 1, "bike" = 2, "bus" = 3)))
x <- s(1:3)
decode(x, kv)
```

Longer dictionaries might be more convenient to read from disk and coerce to keyvalue objects using the data.frame method. 
Dictionaries with a lot of values for every key, can also be specified by a list method:
```{r}
ex <- list(
         fruit  = c("banana", "orange", "kiwi"),
         car    = c("SAAB", "Volvo", "taxi", "truck"),
         animal = c("elephant")
)
knitr::kable(as.keyvalue(ex))
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets_keyvalue.R
\docType{data}
\name{sjukvardsomrade}
\alias{sjukvardsomrade}
\title{Geographical health care areas}
\description{
Key-value codes for health care areas.
}
\details{
\itemize{
  \item{Key: the first four digits from the LKF-code}
  \item{Value: A geographical area (sub area of county/lan) with special
    interest}
}

The object currently only applies to the Western health care region
\itemize{
  \item{Storgoteborg}
  \item{Fyrbodal}
  \item{Sodra_Alvsborg}
  \item{Skaraborg}
  \item{Norra_Halland}
}
Updates for other regions needs to be requested (please do!).
}
\section{Kungalv}{

 Kungalv is an independent area by default. There are situations when Kungalv
 should be dealt with as an
 independent health care region or as a part of greater Gothenburg.
 See section "extra_functions" in \link{decode}
 to handle this.
}

\seealso{
\link{extra_functions}

Other key_value_data:
\code{\link{atc}},
\code{\link{distrikt}},
\code{\link{figo}},
\code{\link{forsamling}},
\code{\link{hemort}},
\code{\link{hsn}},
\code{\link{icd10cm}},
\code{\link{icd10se}},
\code{\link{icd7}},
\code{\link{icd7_grov}},
\code{\link{icd9}},
\code{\link{icd9cmd}},
\code{\link{icdo}},
\code{\link{icdo3}},
\code{\link{icdo3_grov}},
\code{\link{klinik}},
\code{\link{kommun}},
\code{\link{kon}},
\code{\link{kva}},
\code{\link{lan}},
\code{\link{m_rtr}},
\code{\link{n_rtr}},
\code{\link{pad}},
\code{\link{patologiavdelning}},
\code{\link{region}},
\code{\link{rockan}},
\code{\link{sida}},
\code{\link{sjukhus}},
\code{\link{sjukhus_inca}},
\code{\link{sjukhus_par}},
\code{\link{snomed}},
\code{\link{snomed3}},
\code{\link{t_rtr}},
\code{\link{tnmgrund}}
}
\concept{key_value_data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.key.R, R/as.value.R
\name{as.key}
\alias{as.key}
\alias{is.key}
\alias{key_and_value}
\alias{as.value}
\alias{is.value}
\title{Coerce to key or value in objects in a key-value pair}
\usage{
as.key(x)

is.key(x)

as.value(x)

is.value(x)
}
\arguments{
\item{x}{a vector}
}
\value{
\code{as.value} returns a value-object (currently the vector itself 
but with class "value").
\code{as.key} returns a key-object (building on \code{value} but with unique 
entries and as character).
}
\description{
Functions to check if an object is a valid key or value in a keyvalue pair, 
or coerce it if possible.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/extra_functions.R
\name{extra_functions}
\alias{extra_functions}
\alias{kungalv2Storgoteborg}
\alias{kungalv2Fyrbodal}
\alias{real_names}
\alias{short_region_names}
\title{Extra functions to apply to decoded variables using decode}
\usage{
kungalv2Storgoteborg(x)

kungalv2Fyrbodal(x)

real_names(x)

short_region_names(x)
}
\arguments{
\item{x}{The decoded variable to be further processed. 
This argument should not be set by the user.
It is used only internally by the function from which it is called.}
}
\description{
See section "extra_functions" at the \link{decode} help page!
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/code.R, R/decode.R, R/decode.data.frame.R,
%   R/decode.default.R
\name{code}
\alias{code}
\alias{decode}
\alias{decode.data.frame}
\alias{decode.default}
\title{Decode codes to plain text (and vice versa)}
\usage{
code(y, keyvalue, verbose = TRUE)

decode(x, ...)

\method{decode}{data.frame}(x, ...)

\method{decode}{default}(x, keyvalue, extra_functions = NULL, exact = FALSE, ...)
}
\arguments{
\item{y}{value to be coded (to be matched against the \code{value}
element) in a \link{keyvalue} object.}

\item{keyvalue}{either a name (as character string) of a package internal
\link{keyvalue} object, or a user
defined keyvalue object (see \link{as.keyvalue}).}

\item{verbose}{(only for \code{code}) can be set to \code{FALSE} to avoid a
printed message to the console if an error occur (\code{TRUE} as default).}

\item{x}{object to decode. Either a key vector to be matched against the
\code{key} column in \code{keyvalue}, or a data.frame (see section
\code{decode.data.frame}). object.}

\item{...}{ignored}

\item{extra_functions}{is a list of functions (or names of functions as
character vector) to be applied to the decoded data after decoding (see
section "extra_functions" below).}

\item{exact}{Should \code{x} have an exact match from the key? Default is
\code{FALSE}. When \code{FALSE}, \code{x} might be transformed to fit the
key (punctuation might be removed, upper case changed to lower case or vice
versa and strings that are too long might be substringed). (\code{code} only
accept exact matches.)}
}
\value{
\itemize{ \item For default S3 method: A vector of the same length as
 \code{x} but with all cells decoded (or coded) to plain text (or code) as
 character. \item For S3 method for class 'data.frame': Data.frame \code{x}
 is returned, possibly with some extra columns (names ending in
 '_Beskrivning'), decoded from columns with names corresponding to attribute
 \code{standard_var_names} for keyvalue objects listed by
 \code{list_keyvalues()}. }
}
\description{
Translate coded values into meaningful plain text (or reversed).
}
\section{Vignette}{
 See the vignette for a longer introduction to the package:
 \code{vignette("decoder")}
}

\section{decode.data.frame}{
 If \code{x} is a data.frame, all column names of
 \code{x} are matched to attribute \code{standard_var_names} for all keyvalue
 objects in the package (see \code{list_keyvalues()}). If the column name is
 a standard name used for a coding, the corresponding keyvalue object is used
 to decode the column and to add an extra column to \code{x} with its
 original name with suffix \code{_Beskrivning}. This is done for all
 identified columns.
}

\section{extra_functions}{
 The relationship between the key and the value in a
 keyvalue object is either 1:1 or m:1. The mapping is straight forward for
 1:1 but with m:1, different applications might require slightly different
 groupings of the keys. One solution is to have several versions of the
 \link{keyvalue} object. Another (which we prefer) is to use the same
 \link{keyvalue} object (if possible) but to call one or several extra
 function(s) to further process the result. These functions are either built
 in package functions that should be called by quoted names or user defined
 functions that can be called by either quoted or unquoted names (if
 available in the current environment). Note that the order of the functions
 could matter since they are called in turn (the output from the first
 function is passed as input to the second function etc).

 Standard functions and how to use them:

 To use with \link{sjukvardsomrade}:

 \describe{ \item{\code{kungalv2Fyrbodal}}{The default classification used in
 \link{sjukvardsomrade} is to make Kungalv a region of its own. Use this
 function if Kungalv should be included in Fyrbodal. See example section
 below.} \item{\code{kungalv2Storgoteborg}}{As \code{kungalv2Fyrbodal} but
 classifies Kungalv as a part of Storgoteborg.} \item{\code{real_names}}{Give
 the area names with correct Swedish spelling (including spaces). This is not
 as default due to compatibility reasons and because names with spaces must
 be back-ticked when referred to.} }

 To use with \link{region} \describe{
 \item{\code{short_region_names}}{Exclude the prefix 'Region' from the region
 names, hence 'Syd' instead of 'Region Syd' etcetera.} }
}

\examples{

KON_VALUE <- sample(1:2, 20, replace = TRUE)
(kon <- decode(KON_VALUE, decoder::kon))
code(kon, decoder::kon)

# Get a sample of Snomed-codes
snomed2 <- sample(decoder::snomed$key, 30, replace = TRUE)
# ... then decode them:
(snomed3 <- decode(snomed2, "snomed"))


# Health care regions can be defined in more than one way
# By default Kungalv define a region of its own:
set.seed(123456789)
healtcare_areas_west <-
  sample(unlist(decoder::sjukvardsomrade), 100, replace = TRUE)
(areas <- decode(healtcare_areas_west, "sjukvardsomrade"))
table(areas)

# But if we want Kungalv to be a part of Storgoteborg
# (which is common practice for example with lung cancer data):
(areas2 <-
  decode(healtcare_areas_west, "sjukvardsomrade", "kungalv2Storgoteborg"))
table(areas2)

# We can also combine several extra_functions if we for example
# also want the area names with correct Swedish spelling.
(areas3 <-
  decode(
    healtcare_areas_west,
    "sjukvardsomrade", c("kungalv2Storgoteborg", "real_names")
 )
)


# The region names can be both with and without prefix:
regs <- sample(6, 10, replace = TRUE)
decode(regs, "region") # With prefix
decode(regs, "region", "short_region_names") # without prefix

# Note that only the first four digits of the LKF-code were used abowe?
# What if we use the full LKF-code?
lkfs <- sample(decoder::forsamling$key, 100, replace = TRUE)
decode(lkfs, "sjukvardsomrade")
# That work's just as fine when exact = FALSE (which it is by default).

# decode can also be used for data.frames with recognised column names
d <- data.frame(
     kon = sample(1:2, 10, replace = TRUE),
     sex = sample(1:2, 10, replace = TRUE),
     lkf = sample(decoder::hemort$key, 10, replace = TRUE)
 )
 decode(d)
}
\seealso{
\link{keyvalue}, \link{extra_functions}
}
\author{
Erik Bulow
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/internals.R
\name{pad0}
\alias{pad0}
\title{Pad leading zeros to character}
\usage{
pad0(x, n)
}
\arguments{
\item{x}{character or object that can be coerced to such}

\item{n}{desired length of outcome vector}
}
\value{
Character vector of length \code{n}
}
\description{
Pad leading zeros to character
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets_keyvalue.R
\docType{data}
\name{hsn}
\alias{hsn}
\title{HSN code (Hälso- och sjukvårdsnamnd)}
\description{
Key-value codes for the HSN:s in VGR valid from 2015.
Based on municipality codes (\code{\link{kommun}}).
Data from 2014-12-17.
}
\details{
Categorization:
\describe{
\item{Norra}{Lysekil, Munkedal, Orust, Sotenäs, Strömstad, Tanum, Bengtsfors,
Dals-Ed, Färgelanda, Mellerud, Åmål, Trollhättan, Uddevalla och Vänersborg}
\item{Västra}{Lilla Edet, Ale, Kungälv, Stenungsund, Tjörn, Öckerö, Härryda,
Mölndal, Partille, Lerum och Alingsås}
\item{Södra}{Herrljunga, Vårgårda, Bollebygd, Borås, Mark, Svenljunga,
Tranemo och Ulricehamn}
\item{Göteborg}{Göteborg}
\item{Östra}{Essunga, Falköping, Grästorp, Götene, Lidköping, Skara, Vara,
Gullspång, Hjo, Karlsborg, Mariestad, Skövde, Tibro, Tidaholm och Töreboda}
}
}
\seealso{
Other key_value_data:
\code{\link{atc}},
\code{\link{distrikt}},
\code{\link{figo}},
\code{\link{forsamling}},
\code{\link{hemort}},
\code{\link{icd10cm}},
\code{\link{icd10se}},
\code{\link{icd7}},
\code{\link{icd7_grov}},
\code{\link{icd9}},
\code{\link{icd9cmd}},
\code{\link{icdo}},
\code{\link{icdo3}},
\code{\link{icdo3_grov}},
\code{\link{klinik}},
\code{\link{kommun}},
\code{\link{kon}},
\code{\link{kva}},
\code{\link{lan}},
\code{\link{m_rtr}},
\code{\link{n_rtr}},
\code{\link{pad}},
\code{\link{patologiavdelning}},
\code{\link{region}},
\code{\link{rockan}},
\code{\link{sida}},
\code{\link{sjukhus}},
\code{\link{sjukhus_inca}},
\code{\link{sjukhus_par}},
\code{\link{sjukvardsomrade}},
\code{\link{snomed}},
\code{\link{snomed3}},
\code{\link{t_rtr}},
\code{\link{tnmgrund}}
}
\concept{key_value_data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets_keyvalue.R
\docType{data}
\name{region}
\alias{region}
\title{Swedish health care regional codes}
\description{
Key-value codes for the Swedish health care regions (1-6).
}
\seealso{
Other key_value_data:
\code{\link{atc}},
\code{\link{distrikt}},
\code{\link{figo}},
\code{\link{forsamling}},
\code{\link{hemort}},
\code{\link{hsn}},
\code{\link{icd10cm}},
\code{\link{icd10se}},
\code{\link{icd7}},
\code{\link{icd7_grov}},
\code{\link{icd9}},
\code{\link{icd9cmd}},
\code{\link{icdo}},
\code{\link{icdo3}},
\code{\link{icdo3_grov}},
\code{\link{klinik}},
\code{\link{kommun}},
\code{\link{kon}},
\code{\link{kva}},
\code{\link{lan}},
\code{\link{m_rtr}},
\code{\link{n_rtr}},
\code{\link{pad}},
\code{\link{patologiavdelning}},
\code{\link{rockan}},
\code{\link{sida}},
\code{\link{sjukhus}},
\code{\link{sjukhus_inca}},
\code{\link{sjukhus_par}},
\code{\link{sjukvardsomrade}},
\code{\link{snomed}},
\code{\link{snomed3}},
\code{\link{t_rtr}},
\code{\link{tnmgrund}}
}
\concept{key_value_data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.keyvalue.R, R/as.keyvalue.data.frame.R,
%   R/as.keyvalue.default.R, R/is.keyvalue.R
\name{keyvalue}
\alias{keyvalue}
\alias{as.keyvalue}
\alias{as.keyvalue.data.frame}
\alias{as.keyvalue.default}
\alias{is.keyvalue}
\title{Coerce to a keyvalue object}
\usage{
as.keyvalue(x, ...)

\method{as.keyvalue}{data.frame}(x, standard_var_names = NULL, ...)

\method{as.keyvalue}{default}(x, y, ...)

is.keyvalue(x)
}
\arguments{
\item{x}{object to test for, or coerce to, keyvalue (see the details).}

\item{...}{arguments passed between methods}

\item{standard_var_names}{a character vector with standard names for
variables decoded with this key-values.}

\item{y}{a value vector if \code{x} is an unnamed vector (argument must be
named 'y = .')}
}
\value{
\code{as.keyvalue} returns a data.frame with additional class
\code{keyvalue} that fulfills the requirements for a \code{keyvalue} object
and with the \code{key} column as character. It also has two extra attributes
\code{standard_var_names} as described above and \code{keyvalue11} which is
\code{TRUE} for keyvalue objects with a 1:1 relation between its keys and
values (otherwise FALSE).
}
\description{
Functions to check if an object is a keyvalue object, or coerce it if
possible.
}
\details{
\code{x} can be of: \itemize{ \item a name of a package internal (or
external) keyvalue object (character of length one) \item a \code{data.frame}
with two columns "key" (with unique cells) and "value". \item a list as
described in \code{\link{as.keyvalue.list}} \item a named vector with all
names unique \item an unnamed vector with unique cells if supplemented by
additional vector (y) }
}
\examples{
snomed2 <- as.keyvalue(snomed)
is.keyvalue(snomed2)
summary(snomed2)
}
\seealso{
\link{as.keyvalue.list}, \link{decode}, \link{summary.keyvalue},
  \link{key_and_value}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/as.keyvalue.list.R
\name{as.keyvalue.list}
\alias{as.keyvalue.list}
\title{Coerce a list to keyvalue object}
\usage{
\method{as.keyvalue}{list}(x, ...)
}
\arguments{
\item{x}{a \code{list} with properties described in the details section.}

\item{...}{further arguments passed to \code{\link{as.keyvalue}}.}
}
\value{
The function returns an object of class \code{keyvalue} 
(and \code{list}.
}
\description{
\code{x} should be a list with properties described in section "Details".
}
\details{
\itemize{
   \item{All names of the list elements should be unique.}
   \item{All elements of the list should be named.}
   \item{All keys should be unique (a key should only be mappad to 
   one value).}
   \item{All elements of the list should be atomic vectors.}
}
}
\examples{

ex <- list(
 fruit  = c("banana", "orange", "kiwi"),
 car    = c("SAAB", "Volvo", "taxi", "truck"),
 animal = c("elephant")
)
as.keyvalue(ex)
is.keyvalue(ex)

}
\seealso{
\link{as.keyvalue}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets_keyvalue.R
\docType{data}
\name{sjukhus}
\alias{sjukhus}
\title{Hospital codes}
\description{
Key-value codes for Swedish hospitals and some primary health care units.
}
\details{
Data from 'Rockan'. Note that this is an old version of the classification!
See the reference link below for the new version.
The old version should be used of historical reasons.
One difference is for example that the Sahlgrenska university hospital is
one hospital
in the new version of the classification but the Swedish regional cancer
centers
still classify it as three different hospitals.
Note also that primary health care units did receive their own codes until
1992 (?)
but not later!
}
\seealso{
Other key_value_data:
\code{\link{atc}},
\code{\link{distrikt}},
\code{\link{figo}},
\code{\link{forsamling}},
\code{\link{hemort}},
\code{\link{hsn}},
\code{\link{icd10cm}},
\code{\link{icd10se}},
\code{\link{icd7}},
\code{\link{icd7_grov}},
\code{\link{icd9}},
\code{\link{icd9cmd}},
\code{\link{icdo}},
\code{\link{icdo3}},
\code{\link{icdo3_grov}},
\code{\link{klinik}},
\code{\link{kommun}},
\code{\link{kon}},
\code{\link{kva}},
\code{\link{lan}},
\code{\link{m_rtr}},
\code{\link{n_rtr}},
\code{\link{pad}},
\code{\link{patologiavdelning}},
\code{\link{region}},
\code{\link{rockan}},
\code{\link{sida}},
\code{\link{sjukhus_inca}},
\code{\link{sjukhus_par}},
\code{\link{sjukvardsomrade}},
\code{\link{snomed}},
\code{\link{snomed3}},
\code{\link{t_rtr}},
\code{\link{tnmgrund}}
}
\concept{key_value_data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets_keyvalue.R
\docType{data}
\name{pad}
\alias{pad}
\title{PAD (C24) code}
\description{
Key-value codes for PAD (C24) Data from Rockan 2012-09-18.
}
\seealso{
Other key_value_data:
\code{\link{atc}},
\code{\link{distrikt}},
\code{\link{figo}},
\code{\link{forsamling}},
\code{\link{hemort}},
\code{\link{hsn}},
\code{\link{icd10cm}},
\code{\link{icd10se}},
\code{\link{icd7}},
\code{\link{icd7_grov}},
\code{\link{icd9}},
\code{\link{icd9cmd}},
\code{\link{icdo}},
\code{\link{icdo3}},
\code{\link{icdo3_grov}},
\code{\link{klinik}},
\code{\link{kommun}},
\code{\link{kon}},
\code{\link{kva}},
\code{\link{lan}},
\code{\link{m_rtr}},
\code{\link{n_rtr}},
\code{\link{patologiavdelning}},
\code{\link{region}},
\code{\link{rockan}},
\code{\link{sida}},
\code{\link{sjukhus}},
\code{\link{sjukhus_inca}},
\code{\link{sjukhus_par}},
\code{\link{sjukvardsomrade}},
\code{\link{snomed}},
\code{\link{snomed3}},
\code{\link{t_rtr}},
\code{\link{tnmgrund}}
}
\concept{key_value_data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets_keyvalue.R
\docType{data}
\name{klinik}
\alias{klinik}
\title{Clinic codes}
\description{
Key-value codes for the Swedish clinic types.
Data mainly from url below (taken from the PDF).
Some extra codes are added from Rockan
(290, 291, 292, 446, 921 and 999 for unknown).
}
\seealso{
Other key_value_data:
\code{\link{atc}},
\code{\link{distrikt}},
\code{\link{figo}},
\code{\link{forsamling}},
\code{\link{hemort}},
\code{\link{hsn}},
\code{\link{icd10cm}},
\code{\link{icd10se}},
\code{\link{icd7}},
\code{\link{icd7_grov}},
\code{\link{icd9}},
\code{\link{icd9cmd}},
\code{\link{icdo}},
\code{\link{icdo3}},
\code{\link{icdo3_grov}},
\code{\link{kommun}},
\code{\link{kon}},
\code{\link{kva}},
\code{\link{lan}},
\code{\link{m_rtr}},
\code{\link{n_rtr}},
\code{\link{pad}},
\code{\link{patologiavdelning}},
\code{\link{region}},
\code{\link{rockan}},
\code{\link{sida}},
\code{\link{sjukhus}},
\code{\link{sjukhus_inca}},
\code{\link{sjukhus_par}},
\code{\link{sjukvardsomrade}},
\code{\link{snomed}},
\code{\link{snomed3}},
\code{\link{t_rtr}},
\code{\link{tnmgrund}}
}
\concept{key_value_data}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/keyvalue_methods.R
\name{summary.keyvalue}
\alias{summary.keyvalue}
\title{Object summary for keyvalue objects}
\usage{
\method{summary}{keyvalue}(object, ...)
}
\arguments{
\item{object}{a keyvalue object}

\item{...}{ignored}
}
\value{
A list with three elements: a summary of the underlying data.frame,
the standard_var_names and possibly a data.frame with all non 1:1 relations.
}
\description{
Objects of type \code{keyvalue11} are summarized as
data.frames with an additional display of possible
\code{standard_var_names}. Objects of type \code{keyvalue}
also gives a list of non 1:1 key-value pairs.
}
\seealso{
\link{keyvalue}
}
